@@ -52,9 +52,6 @@ class IndexedMzML(TagBase):
 
 class ComponentDispatcher(XMLBindingDispatcherBase):
 
-    _native_id_maker = None
-    native_id_format = None
-
     def __init__(self, *args, **kwargs):
         super(
             ComponentDispatcher,
@@ -70,10 +67,10 @@ class ComponentBase(_ComponentBase):
 
 default_cv_list = [
     CV(id='PSI-MS',
-       uri='https://raw.githubusercontent.com/HUPO-PSI/psi-ms-CV/master/psi-ms.obo',
+       uri='http://purl.obolibrary.org/obo/ms/psi-ms.obo',
        full_name='PSI-MS'),
     CV(id='UO',
-       uri='http://ontologies.berkeleybop.org/uo.obo',
+       uri='http://purl.obolibrary.org/obo/uo.obo',
        full_name='UNIT-ONTOLOGY')
 ]
 
@@ -117,7 +114,23 @@ class IDGenericCollection(GenericCollection):
             member.write(xml_file)
 
 
-class FileContent(ComponentBase):
+class HasNativeIDFormat(object):
+
+    @property
+    def native_id_format(self):
+        formats = []
+        for param in self.params:
+            param = self.context.param(param)
+            try:
+                term = self.context.term(param.accession)
+            except (AttributeError, KeyError):
+                continue
+            if term.is_of_type("MS:1000767"):
+                formats.append(term)
+        return formats
+
+
+class FileContent(ComponentBase, HasNativeIDFormat):
     requires_id = False
 
     def __init__(self, params=None, context=NullMap, **kwargs):
@@ -141,7 +154,7 @@ class SourceFileList(GenericCollection):
         return formats
 
 
-class SourceFile(ComponentBase):
+class SourceFile(ComponentBase, HasNativeIDFormat):
     requires_id = True
 
     def __init__(self, location=None, name=None, id=None,
@@ -189,19 +202,6 @@ class SourceFile(ComponentBase):
     def write_content(self, xml_file):
         self.write_params(xml_file)
 
-    @property
-    def native_id_format(self):
-        formats = []
-        for param in self.params:
-            param = self.context.param(param)
-            try:
-                term = self.context.term(param.accession)
-            except (AttributeError, KeyError):
-                continue
-            if term.is_of_type("MS:1000767"):
-                formats.append(term)
-        return formats
-
 
 class FileDescription(ComponentBase):
     requires_id = False
@@ -227,6 +227,9 @@ class FileDescription(ComponentBase):
 
     @property
     def native_id_formats(self):
+        content_id_format = self.content.native_id_format
+        if content_id_format:
+            return [content_id_format] + self.source_files.native_id_formats
         return self.source_files.native_id_formats
 
     def write_content(self, xml_file):
@@ -630,7 +633,7 @@ class Spectrum(ComponentBase):
             sourceFileRef=self._source_file_reference,
             defaultArrayLength=self.default_array_length,
             dataProcessingRef=self._data_processing_reference,
-            id_formatter=context['native_id_formatter'])
+            id_formatter=context.get('native_id_formatter'))
         self.context = context
         self.context['Spectrum'][id] = self.element.id
         self.params = self.prepare_params(params, **kwargs)
